@@ -1,11 +1,14 @@
+#define DEBUGTIMESTAMPS		FALSE
+
 #include "AppleDBDMAAudioDMAEngine.h"
+#include "AppleOnboardAudio.h"
 
 #include <IOKit/IOMemoryDescriptor.h>
 #include <IOKit/audio/IOAudioDevice.h>
 #include <IOKit/audio/IOAudioTypes.h>
 #include <IOKit/audio/IOAudioDefines.h>
 #include <IOKit/audio/IOAudioDebug.h>
-#include <IOKit/audio/IOAudioToggleControl.h>
+//#include <IOKit/audio/IOAudioToggleControl.h>
 #include <IOKit/audio/IOAudioLevelControl.h>
 
 #include <IOKit/IOFilterInterruptEventSource.h>
@@ -26,52 +29,58 @@ const int AppleDBDMAAudioDMAEngine::kDBDMAOutputIndex	= 1;
 const int AppleDBDMAAudioDMAEngine::kDBDMAInputIndex	= 2;
 
 bool AppleDBDMAAudioDMAEngine::init(OSDictionary	*properties,
-                                 IOService 	*theDeviceProvider,
-                                 bool		hasInput,
-                                 UInt32		nBlocks,
-                                 UInt32		bSize,
-                                 UInt32		rate,
-                                 UInt16		bitDepth,
-                                 UInt16		numChannels)
+                                 IOService 			*theDeviceProvider,
+                                 bool				hasInput,
+                                 UInt32				nBlocks,
+                                 UInt32				bSize,
+                                 UInt32				rate,
+                                 UInt16				bitDepth,
+                                 UInt16				numChannels)
 {
-    IOAudioSampleRate initialSampleRate;
-    IOMemoryMap *map;
-    
-    CLOG("+ AppleDBDMAAudioDMAEngine::init\n");
-            //Ususal check
-    if (!super::init(properties)) {return false;}
-    if (!theDeviceProvider) {return false;}
+	IOAudioSampleRate initialSampleRate;
+	IOMemoryMap *map;
+	Boolean					result;
 
-            //create the memory places for DMA
-    map = theDeviceProvider->mapDeviceMemoryWithIndex(AppleDBDMAAudioDMAEngine::kDBDMAOutputIndex);
-    if(!map) { return false;}
-    ioBaseDMAOutput = (IODBDMAChannelRegisters *) map->getVirtualAddress();
-    
-    if(hasInput) {
-        map = theDeviceProvider->mapDeviceMemoryWithIndex(AppleDBDMAAudioDMAEngine::kDBDMAInputIndex);
-        if(!map) { return false;}
-        ioBaseDMAInput = (IODBDMAChannelRegisters *) map->getVirtualAddress();
-    } else {
-        ioBaseDMAInput = 0;
-    }
-    
-    dmaCommandBufferIn = 0;
-    dmaCommandBufferOut = 0;
-    commandBufferSize = 0;
-    interruptEventSource = 0;
+	CLOG("+ AppleDBDMAAudioDMAEngine::init\n");
+	result = FALSE;
 
-    numBlocks = nBlocks;
-    blockSize = bSize;
-    setSampleOffset(kMinimumLatency);
-    setNumSampleFramesPerBuffer(numBlocks * blockSize / 4);
-    
-    initialSampleRate.whole = rate;
-    initialSampleRate.fraction = 0;
-    
-    setSampleRate(&initialSampleRate);
+	// Ususal check
+	FailIf (FALSE == super::init (properties), Exit);
+	FailIf (NULL == theDeviceProvider, Exit);
 
-    CLOG("- AppleDBDMAAudioDMAEngine::init\n");    
-    return true;
+	// create the memory places for DMA
+	map = theDeviceProvider->mapDeviceMemoryWithIndex(AppleDBDMAAudioDMAEngine::kDBDMAOutputIndex);
+	FailIf (NULL == map, Exit);
+	ioBaseDMAOutput = (IODBDMAChannelRegisters *) map->getVirtualAddress();
+
+	if(hasInput) {
+		map = theDeviceProvider->mapDeviceMemoryWithIndex(AppleDBDMAAudioDMAEngine::kDBDMAInputIndex);
+		FailIf (NULL == map, Exit);
+		ioBaseDMAInput = (IODBDMAChannelRegisters *) map->getVirtualAddress();
+	} else {
+		ioBaseDMAInput = 0;
+	}
+
+	dmaCommandBufferIn = 0;
+	dmaCommandBufferOut = 0;
+	commandBufferSize = 0;
+	interruptEventSource = 0;
+
+	numBlocks = nBlocks;
+	blockSize = bSize;
+	setSampleOffset(kMinimumLatency);
+	setNumSampleFramesPerBuffer(numBlocks * blockSize / 4);
+
+	initialSampleRate.whole = rate;
+	initialSampleRate.fraction = 0;
+
+	setSampleRate(&initialSampleRate);
+
+	result = TRUE;
+
+Exit:
+	CLOG("- AppleDBDMAAudioDMAEngine::init\n");    
+	return result;
 }
 
 void AppleDBDMAAudioDMAEngine::free()
@@ -134,7 +143,7 @@ bool AppleDBDMAAudioDMAEngine::initHardware(IOService *provider)
     ourProvider = provider;
     fBadCmd = 0;
     fBadResult = 0;
-	
+
     if (!super::initHardware(provider)) {return false;}
         
         //allocate the memory for the buffer
@@ -414,9 +423,8 @@ IOReturn AppleDBDMAAudioDMAEngine::message (UInt32 type, IOService * provider, v
 
 IOReturn AppleDBDMAAudioDMAEngine::performAudioEngineStart()
 {
-    DEBUG_IOLOG(" + AppleDBDMAAudioDMAEngine::performAudioEngineStart()\n");
+    debugIOLog(" + AppleDBDMAAudioDMAEngine::performAudioEngineStart()\n");
 
-    
     if (!ioBaseDMAOutput || !dmaCommandBufferOut || !status || !interruptEventSource) {
         return kIOReturnError;
     }
@@ -455,7 +463,7 @@ IOReturn AppleDBDMAAudioDMAEngine::performAudioEngineStart()
     IOSetDBDMABranchSelect(ioBaseDMAOutput, IOSetDBDMAChannelControlBits(kdbdmaS0));
     IODBDMAStart(ioBaseDMAOutput, (IODBDMADescriptor *)pmap_extract(kernel_pmap, (vm_address_t)(dmaCommandBufferOut)));
 
-    DEBUG_IOLOG(" - AppleDBDMAAudioDMAEngine::performDMAEngineStart()\n");
+    debugIOLog(" - AppleDBDMAAudioDMAEngine::performAudioEngineStart()\n");
     return kIOReturnSuccess;
 }
 
@@ -466,15 +474,6 @@ IOReturn AppleDBDMAAudioDMAEngine::restartOutputIfFailure(){
 
     flush_dcache((vm_offset_t)dmaCommandBufferOut, commandBufferSize, false);
 
-    filterState.xl_1 = 0.0;
-    filterState.xr_1 = 0.0;
-    filterState.xl_2 = 0.0;
-    filterState.xr_2 = 0.0;
-    filterState.yl_1 = 0.0;
-    filterState.yr_1 = 0.0;
-    filterState.yl_2 = 0.0;
-    filterState.yr_2 = 0.0;
-
     if (NULL != iSubEngine) {
 		startiSub = TRUE;
 		needToSync = TRUE;
@@ -482,7 +481,7 @@ IOReturn AppleDBDMAAudioDMAEngine::restartOutputIfFailure(){
 
     interruptEventSource->enable();
 
-        //add the time stamp take to test
+	// add the time stamp take to test
     takeTimeStamp(false);
     
     IOSetDBDMAChannelControl(ioBaseDMAOutput, IOClearDBDMAChannelControlBits(kdbdmaS0));
@@ -491,16 +490,13 @@ IOReturn AppleDBDMAAudioDMAEngine::restartOutputIfFailure(){
                                 (vm_address_t)(dmaCommandBufferOut)));
 
     return kIOReturnSuccess;
-
-
 }
 
 IOReturn AppleDBDMAAudioDMAEngine::performAudioEngineStop()
 {
     UInt16 attemptsToStop = 1000;
 
-
-    DEBUG2_IOLOG("+ AppleDBDMAAudioDMAEngine[%p]::performAudioEngineStop()\n", this);
+    debugIOLog("+ AppleDBDMAAudioDMAEngine::performAudioEngineStop()\n");
 
     if (NULL != iSubEngine) {
         iSubEngine->StopiSub ();
@@ -513,7 +509,7 @@ IOReturn AppleDBDMAAudioDMAEngine::performAudioEngineStop()
 
     interruptEventSource->disable();
         
-        //stop the output
+	// stop the output
     IOSetDBDMAChannelControl(ioBaseDMAOutput, IOSetDBDMAChannelControlBits(kdbdmaS0));
     while ((IOGetDBDMAChannelStatus(ioBaseDMAOutput) & kdbdmaActive) && (attemptsToStop--)) {
         eieio();
@@ -523,7 +519,7 @@ IOReturn AppleDBDMAAudioDMAEngine::performAudioEngineStop()
     IODBDMAStop(ioBaseDMAOutput);
     IODBDMAReset(ioBaseDMAOutput);
 
-        //stop the input
+	// stop the input
     if(ioBaseDMAInput){
         IOSetDBDMAChannelControl(ioBaseDMAInput, IOSetDBDMAChannelControlBits(kdbdmaS0));
         while ((IOGetDBDMAChannelStatus(ioBaseDMAInput) & kdbdmaActive) && (attemptsToStop--)) {
@@ -536,11 +532,11 @@ IOReturn AppleDBDMAAudioDMAEngine::performAudioEngineStop()
     }
     
     interruptEventSource->enable();
+
     DEBUG_IOLOG("- AppleDBDMAAudioDMAEngine::performAudioEngineStop()\n");
     return kIOReturnSuccess;
 }
 
-
 bool AppleDBDMAAudioDMAEngine::filterInterrupt(int index)
 {
 	// check to see if this interupt is because the DMA went bad
@@ -597,18 +593,47 @@ UInt32 AppleDBDMAAudioDMAEngine::getCurrentSampleFrame()
     return currentBlock * blockSize / 4;	// 4 bytes per frame - 2 per sample * 2 channels - BIG HACK
 }
 
+// This gets called when a new audio stream needs to be mixed into an already playing audio stream
 void AppleDBDMAAudioDMAEngine::resetClipPosition (IOAudioStream *audioStream, UInt32 clipSampleFrame) {
-	iSubBufferOffset -= (previousClippedToFrame - clipSampleFrame) * 2;		// should be * streamFormat->fNumChannels, but that's not readily available
-	previousClippedToFrame = clipSampleFrame;
+    if (NULL != iSubBufferMemory) {
+		// start the filter over again since old filter state is invalid
+		filterState.xl_1 = 0.0;
+		filterState.xr_1 = 0.0;
+		filterState.xl_2 = 0.0;
+		filterState.xr_2 = 0.0;
+		filterState.yl_1 = 0.0;
+		filterState.yr_1 = 0.0;
+		filterState.yl_2 = 0.0;
+		filterState.yr_2 = 0.0;
+
 #if DEBUGLOG
-	IOLog ("resetClipPosition, iSubBufferOffset=%ld, previousClippedToFrame=%ld\n", iSubBufferOffset, previousClippedToFrame);
+		IOLog ("resetClipPosition, iSubBufferOffset=%ld, previousClippedToFrame=%ld, clipSampleFrame=%ld\n", iSubBufferOffset, previousClippedToFrame, clipSampleFrame);
 #endif
+		if (previousClippedToFrame < clipSampleFrame) {
+			// ((numBlocks * blockSize) / 4) is the number of samples in the buffer
+			iSubBufferOffset -= (previousClippedToFrame + (((numBlocks * blockSize) / 4) - clipSampleFrame)) * 2;
+		} else {
+			iSubBufferOffset -= (previousClippedToFrame - clipSampleFrame) * 2;		// should be * streamFormat->fNumChannels, but that's not readily available
+		}
+
+		if (iSubBufferOffset < 0) {
+			iSubBufferOffset += (iSubBufferMemory->getLength () / 2);
+		}
+		previousClippedToFrame = clipSampleFrame;
+#if DEBUGLOG
+		IOLog ("now: iSubBufferOffset=%ld, previousClippedToFrame=%ld\n", iSubBufferOffset, previousClippedToFrame);
+#endif
+    }
 }
 
 extern "C" {
 UInt32 CalculateOffset (UInt64 nanoseconds, UInt32 sampleRate);
 IOReturn clipAppleDBDMAToOutputStream(const void *mixBuf, void *sampleBuf, UInt32 firstSampleFrame, UInt32 numSampleFrames, const IOAudioStreamFormat *streamFormat);
+IOReturn clipAppleDBDMAToOutputStreamInvertRightChannel(const void *mixBuf, void *sampleBuf, UInt32 firstSampleFrame, UInt32 numSampleFrames, const IOAudioStreamFormat *streamFormat);
+IOReturn clipAppleDBDMAToOutputStreamMixRightChannel(const void *mixBuf, void *sampleBuf, UInt32 firstSampleFrame, UInt32 numSampleFrames, const IOAudioStreamFormat *streamFormat);
 IOReturn clipAppleDBDMAToOutputStreamiSub(const void *mixBuf, void *sampleBuf, PreviousValues *filterState, float *low, float *high, UInt32 firstSampleFrame, UInt32 numSampleFrames, UInt32 sampleRate, const IOAudioStreamFormat *streamFormat, SInt16 * iSubBufferMemory, UInt32 *loopCount, SInt32 *iSubBufferOffset, UInt32 iSubBufferLen);
+IOReturn clipAppleDBDMAToOutputStreamiSubInvertRightChannel(const void *mixBuf, void *sampleBuf, PreviousValues *filterState, float *low, float *high, UInt32 firstSampleFrame, UInt32 numSampleFrames, UInt32 sampleRate, const IOAudioStreamFormat *streamFormat, SInt16 * iSubBufferMemory, UInt32 *loopCount, SInt32 *iSubBufferOffset, UInt32 iSubBufferLen);
+IOReturn clipAppleDBDMAToOutputStreamiSubMixRightChannel(const void *mixBuf, void *sampleBuf, PreviousValues *filterState, float *low, float *high, UInt32 firstSampleFrame, UInt32 numSampleFrames, UInt32 sampleRate, const IOAudioStreamFormat *streamFormat, SInt16 * iSubBufferMemory, UInt32 *loopCount, SInt32 *iSubBufferOffset, UInt32 iSubBufferLen);
 IOReturn convertAppleDBDMAFromInputStream(const void *sampleBuf, void *destBuf, UInt32 firstSampleFrame, UInt32 numSampleFrames, const IOAudioStreamFormat *streamFormat);
 };
 
@@ -620,9 +645,24 @@ IOReturn AppleDBDMAAudioDMAEngine::clipOutputSamples(const void *mixBuf, void *s
 	UInt32						iSubBufferLen = 0;
 	UInt32						sampleRate;
 
+#if DEBUGTIMESTAMPS
+	AbsoluteTime					currentTime;
+	static AbsoluteTime				lastLoopTime = {0, 0};
+	AbsoluteTime					diff;
+	UInt64						nanos;
+
+	clock_get_uptime (&currentTime);
+	diff.hi = currentTime.hi;
+	diff.lo = currentTime.lo;
+	SUB_ABSOLUTETIME (&diff, &lastLoopTime);
+	lastLoopTime.hi = currentTime.hi;
+	lastLoopTime.lo = currentTime.lo;
+	absolutetime_to_nanoseconds (diff, &nanos);
+	IOLog ("delta = %ld\n", (UInt32)(nanos / (1000 * 1000)));
+#endif
 	// if the DMA went bad restart it
 	if (fBadCmd && fBadResult)
-	{		
+	{
 		fBadCmd = 0;
 		fBadResult = 0;
                 restartOutputIfFailure();
@@ -668,36 +708,50 @@ IOReturn AppleDBDMAAudioDMAEngine::clipOutputSamples(const void *mixBuf, void *s
 		}
 		previousClippedToFrame = firstSampleFrame + numSampleFrames;
 
-#if 0
-		if (iSubBufferOffset > (iSubEngine->GetCurrentByteCount () / 2)) {
-			iSubLead = iSubBufferOffset - (iSubEngine->GetCurrentByteCount () / 2);
-		} else {
-			iSubLead = iSubBufferLen - (iSubEngine->GetCurrentByteCount () / 2) + iSubBufferOffset;
-		}
-//		IOLog ("iSubLead=%ld\n", iSubLead);
-
-		if (FALSE == needToSync && (iSubEngine->GetCurrentLoopCount () > iSubLoopCount || iSubLead < (initialiSubLead / 2))) {
+		// Detect being out of sync with the iSub
+		if (needToSync == FALSE && 0xFFFFFFFF != iSubEngine->GetCurrentLoopCount ()) {
+		    if (iSubLoopCount > (iSubEngine->GetCurrentLoopCount () + 1) && !(iSubBufferOffset < (SInt32)(iSubEngine->GetCurrentByteCount () / 2))) {
 #if DEBUGLOG
-			IOLog ("iSubLoopCount=%ld, iSubCurrentLoopCount=%ld, iSubCurrentByteCount=%ld, iSubBufferOffset=%ld\n", iSubLoopCount, iSubEngine->GetCurrentLoopCount (), iSubEngine->GetCurrentByteCount (), iSubBufferOffset);
+			IOLog ("****looped more than the iSub iSubLoopCount = %ld, iSubEngine->GetCurrentLoopCount () = %ld\n", iSubLoopCount, iSubEngine->GetCurrentLoopCount ());
 #endif
-			needToSync = true;
+			needToSync = TRUE;
+			startiSub = TRUE;
+		    } else if (iSubLoopCount < iSubEngine->GetCurrentLoopCount ()) {
+#if DEBUGLOG
+			IOLog ("****iSub is ahead of us iSubLoopCount = %ld, iSubEngine->GetCurrentLoopCount () = %ld\n", iSubLoopCount, iSubEngine->GetCurrentLoopCount ());
+#endif
+			needToSync = TRUE;
+			startiSub = TRUE;
+		    } else if (iSubLoopCount == iSubEngine->GetCurrentLoopCount () && iSubBufferOffset < (SInt32)(iSubEngine->GetCurrentByteCount () / 2)) {
+#if DEBUGLOG
+			IOLog ("****iSub is in front of our write head iSubBufferOffset = %ld, (iSubEngine->GetCurrentByteCount () / 2) = %ld\n", iSubBufferOffset, iSubEngine->GetCurrentByteCount () / 2);
+#endif
+			needToSync = TRUE;
+			startiSub = TRUE;
+		    }
 		}
-#endif
 
 		// sync up with iSub
 		if (TRUE == needToSync) {
 			needToSync = FALSE;
+			// start the filter over again since old filter state is invalid
+			filterState.xl_1 = 0.0;
+			filterState.xr_1 = 0.0;
+			filterState.xl_2 = 0.0;
+			filterState.xr_2 = 0.0;
+			filterState.yl_1 = 0.0;
+			filterState.yr_1 = 0.0;
+			filterState.yl_2 = 0.0;
+			filterState.yr_2 = 0.0;
 			iSubLoopCount = iSubEngine->GetCurrentLoopCount ();
-//			if (0 == initialiSubLead) {
-				iSubBufferOffset = (firstSampleFrame - getCurrentSampleFrame ()) * streamFormat->fNumChannels;
-//				initialiSubLead = iSubBufferOffset;
-//			} else {
-//				iSubBufferOffset += (firstSampleFrame - getCurrentSampleFrame () + (iSubEngine->GetCurrentByteCount () / 4)) * streamFormat->fNumChannels;
-//			}
+			iSubBufferOffset = (firstSampleFrame - getCurrentSampleFrame ()) * streamFormat->fNumChannels;
 #if DEBUGLOG
-			IOLog ("starting iSubBufferOffset = %ld, iSubLoopCount = %ld\n", iSubBufferOffset, iSubLoopCount);
+			IOLog ("firstSampleFrame = %ld, getCurrentSampleFrame () = %ld\n", firstSampleFrame, getCurrentSampleFrame ());
 #endif
 			iSubBufferOffset -= 88 * 5;
+#if DEBUGLOG
+			IOLog ("starting iSubBufferOffset = %ld, iSubLoopCount = %ld, numSampleFrames = %ld\n", iSubBufferOffset, iSubLoopCount, numSampleFrames);
+#endif
 		}
 
 		if (iSubBufferOffset > (SInt32)iSubBufferLen) {
@@ -705,7 +759,7 @@ IOReturn AppleDBDMAAudioDMAEngine::clipOutputSamples(const void *mixBuf, void *s
 			iSubLoopCount += iSubBufferOffset / iSubBufferLen;
 			iSubBufferOffset = iSubBufferOffset % iSubBufferLen;
 #if DEBUGLOG
-			IOLog ("iSubBufferOffset > iSubBufferLen, iSubBufferOffset is now %ld\n", iSubBufferOffset);
+			IOLog ("iSubBufferOffset > iSubBufferLen (%ld), iSubBufferOffset is now %ld\n", iSubBufferLen, iSubBufferOffset);
 #endif
 		} else if (iSubBufferOffset < 0) {
 			iSubBufferOffset += iSubBufferLen;
@@ -714,14 +768,30 @@ IOReturn AppleDBDMAAudioDMAEngine::clipOutputSamples(const void *mixBuf, void *s
 #endif
 		}
 
-		result = clipAppleDBDMAToOutputStreamiSub(mixBuf, sampleBuf, &filterState, lowFreqSamples, highFreqSamples, firstSampleFrame, numSampleFrames, sampleRate, streamFormat, (SInt16*)iSubBuffer, &iSubLoopCount, &iSubBufferOffset, iSubBufferLen);
+         // this will be true on a slot load iMac that has the built in speakers enabled
+		if (TRUE == fNeedsPhaseInversion) {
+            result = clipAppleDBDMAToOutputStreamiSubInvertRightChannel(mixBuf, sampleBuf, &filterState, lowFreqSamples, highFreqSamples, firstSampleFrame, numSampleFrames, sampleRate, streamFormat, (SInt16*)iSubBuffer, &iSubLoopCount, &iSubBufferOffset, iSubBufferLen);
+		} else if (TRUE == fNeedsRightChanMixed) {
+            result = clipAppleDBDMAToOutputStreamiSubMixRightChannel(mixBuf, sampleBuf, &filterState, lowFreqSamples, highFreqSamples, firstSampleFrame, numSampleFrames, sampleRate, streamFormat, (SInt16*)iSubBuffer, &iSubLoopCount, &iSubBufferOffset, iSubBufferLen);
+		} else {
+            result = clipAppleDBDMAToOutputStreamiSub(mixBuf, sampleBuf, &filterState, lowFreqSamples, highFreqSamples, firstSampleFrame, numSampleFrames, sampleRate, streamFormat, (SInt16*)iSubBuffer, &iSubLoopCount, &iSubBufferOffset, iSubBufferLen);
+		}
+            
 		if (TRUE == startiSub) {
 			iSubEngine->StartiSub ();
 			startiSub = FALSE;
+			iSubLoopCount = 0;
 		}
 	} else {
-		result = clipAppleDBDMAToOutputStream(mixBuf, sampleBuf, firstSampleFrame, numSampleFrames, streamFormat);
-	}
+		// this will be true on a slot load iMac that has the built in speakers enabled
+		if (TRUE == fNeedsPhaseInversion) {
+			result = clipAppleDBDMAToOutputStreamInvertRightChannel(mixBuf, sampleBuf, firstSampleFrame, numSampleFrames, streamFormat);
+		} else if (TRUE == fNeedsRightChanMixed) {
+			result = clipAppleDBDMAToOutputStreamMixRightChannel(mixBuf, sampleBuf, firstSampleFrame, numSampleFrames, streamFormat);
+		} else {
+			result = clipAppleDBDMAToOutputStream(mixBuf, sampleBuf, firstSampleFrame, numSampleFrames, streamFormat);
+		}
+    }
 
 	return result;
 }
@@ -876,6 +946,22 @@ IOReturn AppleDBDMAAudioDMAEngine::iSubOpenAction (OSObject *owner, void *arg1,
 }
 
 /*
+#if 0
+		if (iSubBufferOffset > (iSubEngine->GetCurrentByteCount () / 2)) {
+			iSubLead = iSubBufferOffset - (iSubEngine->GetCurrentByteCount () / 2);
+		} else {
+			iSubLead = iSubBufferLen - (iSubEngine->GetCurrentByteCount () / 2) + iSubBufferOffset;
+		}
+//		IOLog ("iSubLead=%ld\n", iSubLead);
+
+		if (FALSE == needToSync && (iSubEngine->GetCurrentLoopCount () > iSubLoopCount || iSubLead < (initialiSubLead / 2))) {
+#if DEBUGLOG
+			IOLog ("iSubLoopCount=%ld, iSubCurrentLoopCount=%ld, iSubCurrentByteCount=%ld, iSubBufferOffset=%ld\n", iSubLoopCount, iSubEngine->GetCurrentLoopCount (), iSubEngine->GetCurrentByteCount (), iSubBufferOffset);
+#endif
+			needToSync = true;
+		}
+#endif
+
 	Pulled out of clipOutputSamples
 	AbsoluteTime				deltaTime;
 	UInt64						nanoseconds;
